=head1 NAME

Slapd::Backend::Cyrus::Base - base class for mail and user

=head1 DESCRIPTION

Synchronization between LDAP and Cyrus.

=cut

package Slapd::Backend::Cyrus::Base;

use strict;
use warnings;
use base 'Slapd::Backend::Cyrus';

use BerkeleyDB;
use Net::LDAP::Constant qw( LDAP_SUCCESS );

__PACKAGE__->mk_accessors(qw/ cyradm env domain_db usermail_db domainuser_db /);

=head1 PROTECTED METHODS

=over

=item B<_init>()

Purpose:
  * Initialization

Arguments:
  * None

Returns:
  * Nonzero if initialization failed

=cut

sub _init
{
	my $self = shift;

	my $code = $self->SUPER::_init();
	return $code if $code != LDAP_SUCCESS;

	my $env = $self->store->can('env') ? $self->store->env
		: new BerkeleyDB::Env(
			-Home  => $self->{db_home},
			-Flags => DB_CREATE| DB_INIT_CDB | DB_INIT_MPOOL,
		);
	$self->env = $env;

	$self->domain_db = new BerkeleyDB::Hash(
		-Env      => $env,
		-Filename => "domain.db",
		-Flags    => DB_CREATE,
	) or die "Can't open file domain.db: $! $BerkeleyDB::Error\n";

	$self->usermail_db = new BerkeleyDB::Hash(
		-Env      => $env,
		-Filename => "usermail.db",
		-Flags    => DB_CREATE,
	) or die "Can't open file usermail.db: $! $BerkeleyDB::Error\n";

	$self->domainuser_db = new BerkeleyDB::Hash(
		-Env      => $env,
		-Filename => "domainuser.db",
		-Flags    => DB_CREATE,
		-Property => DB_DUP,
	) or die "Can't open file domainuser.db: $! $BerkeleyDB::Error\n";

	$self->usermail_db->associate(
		$self->domainuser_db,
		sub {
			my( $key, $data ) = @_;
			my @domain;
			foreach my $domain ( split /\s+/, $data )
			{
				$domain =~ s/^.*@//;
				push @domain, $domain if $domain;
			}
			if( @domain )
			{
				$_[2] = \@domain; # Third arg is $skey
				return 0;
			}
			else
			{
				return DB_DONOTINDEX;
			}
		}
	);

	return LDAP_SUCCESS;
}

=item B<_config>( $name, @values )

Purpose:
  * Config

Arguments:
  * Scalar - name
  * List - values

Returns:
  * Scalar - result code

=cut

sub _config
{
	my $self = shift;
	my( $name, $value ) = @_;
	if( $name eq 'cyrusDbHome' )
	{
		$self->{db_home} = $value;
	}
	else
	{
		return $self->SUPER::_config(@_);
	}
	return LDAP_SUCCESS;
}

=back

=cut

1;

=head1 SEE ALSO

L<Slapd::Backend::Cyrus>, L<Slapd::Backend::Proxy>, L<slapd-perl>

=head1 AUTHOR

Aleksey Mashanov <aleksey.mashanov@gmail.com>

=head1 LICENSE

This library is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut
